# -*- coding: utf-8 -*-
import smtplib

from django.conf import settings

from email_filter.models import EmailRedirect, get_name_from_email


def notify_unknown_user(
        sender, instance, original_msg, mail_sender_email, send_email, smtp_sendmail, **kwargs):
    smtp_server = smtplib.SMTP(
        settings.EMAIL_HOST, settings.EMAIL_PORT)
    email_from = 'noreply%s' % EmailRedirect.get_server_email_domain()
    msg_format = (
        'To: {sender}\nFrom: {email_from}\nSubject: undelivered mail \n\n'
        'User with username {username} not exists! \n\n')
    sender = sender_email or original_msg['From']
    username_to = get_name_from_email(instance.recipient)
    msg = msg_format.format(
        sender=sender,
        email_from=email_from,
        username=username_to)
    smtp_server.sendmail(email_from, sender, msg)
    smtp_server.quit()



# -*- coding: utf-8 -*-
from email_filter.models import is_project_recipient


def is_outgoing(msg):
    recipients = [msg['To']]
    if recipients and ',' in recipients[0]:
        recipients = recipients[0].split(',')
    for recipient in recipients:
        if is_project_recipient(recipient.strip()):
            return False
    return True


def get_attachments(msg):
    """
    Return list of (filename, file_obj) tuples for attachments of msg.

    excepts ``msg`` to be instance of ``email.message.Message``

    """
    attaches = []
    for part in msg.walk():
        if part.get("Content-Disposition"):
            filename = part.get_filename()
            data = part.get_payload(decode=True)
            content_type = part.get_content_type()
            attaches.append((filename, data, content_type))
    return attaches



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import smtplib
import sys
import os
import email
import logging
import traceback
import cgitb
import uuid
import pytils
cgitb.enable(format='text')

try:
    from mail_route_settings import SELF_ADDRESS, PROCESS_OUTGOING_EMAILS
except ImportError:
    SELF_ADDRESS = 'proxy@kavahq.com'
    PROCESS_OUTGOING_EMAILS = False

PROJECT_NAME = None

if __name__ == "__main__":
    try:
        from mail_route_settings import PROJECT_BASE_DIR, DJANGO_SETTINGS_MODULE, LOG_PATH, PROJECT_NAME
        sys.path.extend([PROJECT_BASE_DIR])
        os.environ['DJANGO_SETTINGS_MODULE'] = DJANGO_SETTINGS_MODULE
        # requires this to open SQLite database as if full path was given
        os.chdir(PROJECT_BASE_DIR)
    except ImportError:
        print("Can't import mail router settings")
        sys.exit(os.EX_CONFIG)


# This imports rely on DJANGO_SETTINGS_MODULE env variable, so do not move them to top
# thirdparty:
from annoying.functions import get_object_or_None

# Django:
from django.shortcuts import _get_queryset
from django.contrib.auth.models import User
from django.core.files.base import ContentFile

# Local:
from email_filter.models import (
    EmailLog, EmailRedirect, get_name_from_email,
    get_cleaned_email, is_project_recipient, is_local_email,
    EmailAttachment,
)
from email_filter.utils import is_outgoing, get_attachments
from email_filter.signals import mail_for_unknown_user

# check django settings:
from django.conf import settings
if PROJECT_NAME:
    from django.utils.importlib import import_module
    import_module(PROJECT_NAME)

# import all apps to get proper signal processing:
for app in settings.INSTALLED_APPS:
    try:
        import_module('%s.models' % app)
    except ImportError:
        pass


logger = logging.getLogger('mail_script')
if __name__ == "__main__":
    if not LOG_PATH:
        LOG_PATH = os.path.join(PROJECT_BASE_DIR, 'mail_route.log')
    logger.addHandler(logging.FileHandler(LOG_PATH))


def get_first_object_or_None(klass, *args, **kwargs):
    """
    Uses get() to return an object or None if the object does not exist.

    klass may be a Model, Manager, or QuerySet object. All other passed
    arguments and keyword arguments are used in the get() query.
    """
    queryset = _get_queryset(klass)
    try:
        return queryset.filter(*args, **kwargs)[0]
    except (queryset.model.DoesNotExist, IndexError):
        return None


def send_them(email_obj, msg, send_email):
    orig_from = msg['From']
    orig_to = msg['To']
    recipient = None
    sender = None
    bcc = []
    username_to = get_name_from_email(email_obj.recipient)
    user_to = get_object_or_None(
        User, username=username_to)
    user_from = get_first_object_or_None(
        User, email=get_cleaned_email(email_obj.sender))

    if 'BCC' in msg:
        bcc = msg['BCC'].split(',')
        bcc = [x.strip() for x in bcc]

    if user_to:
        profile_to = user_to.get_profile()
        if profile_to and profile_to.bcc:
            bcc.append(profile_to.bcc)
    if user_from:
        profile_from = user_from.get_profile()
        if profile_from and profile_from.bcc and profile_from.bcc not in bcc:
            bcc.append(profile_from.bcc)

    if user_to is not None and user_from is None:
        recipient = user_to.email
        sender = EmailRedirect.objects.get_or_create_back_email(
            get_cleaned_email(email_obj.sender))

    if user_from is not None and user_to is None:
        recipient = EmailRedirect.objects.get_original_from_back_email(
            get_cleaned_email(email_obj.recipient))
        sender = '%s%s' % (
            user_from.username, EmailRedirect.get_server_email_domain())

    if user_to is not None and user_from is not None:
        recipient = user_to.email
        sender = '%s%s' % (
            user_from.username, EmailRedirect.get_server_email_domain())

    if not recipient:
        smtp_server = smtplib.SMTP(
            settings.EMAIL_HOST, settings.EMAIL_PORT)
        mail_for_unknown_user.send(
            sender=EmailLog, instance=email_obj, original_msg=msg,
            mail_sender_email=sender,
            send_email=send_email, smtp_sendmail=smtp_server.sendmail
        )
        smtp_server.quit()
        return None

    if recipient is None or sender is None:
        raise NotImplementedError("Sender (%s) or recipient (%s) is not set" % (
            sender, recipient))

    del msg['From']
    del msg['To']
    del msg['Reply-To']
    msg['From'] = sender
    msg['To'] = recipient

    logger.debug(
        '\nin : %35s --> %-35s'
        '\nout: %35s --> %-35s\n' % (
            orig_from, orig_to, msg['From'], msg['To'])
    )

    if 'BCC' in msg:
        del msg['BCC']
    # Send the message via local SMTP server.
    if send_email and user_to.email and not is_local_email(user_to.email):
        smtp_server = smtplib.SMTP(settings.EMAIL_HOST, settings.EMAIL_PORT)
        if settings.EMAIL_USE_TLS:
            raise NotImplementedError("using TLS is not supported")
        try:
            smtp_server.sendmail(sender, recipient, msg.as_string())
            if bcc:
                msg['BCC'] = ','.join(bcc)
                smtp_server.sendmail(sender, bcc, msg.as_string())
        except smtplib.SMTPRecipientsRefused as e:
            logger.error("\nCan't send mail to recipients: %s. Original recipients: %s" % (
                e.recipients, orig_to))
            logger.debug(traceback.format_exc())
            raise
        except Exception as e:
            logger.error('\nUnknown error.')
            logger.debug(traceback.format_exc())
            raise
        smtp_server.quit()
    else:
        msg['BCC'] = ','.join(bcc)
    return msg


def save_email_object(raw_email, send_email=False):

    parser = email.FeedParser.FeedParser()
    for msg_line in raw_email:
        parser.feed(msg_line)
    msg = parser.close()

    if is_outgoing(msg) and not PROCESS_OUTGOING_EMAILS:
        return

    if 'BCC' in msg:
        bcc = msg['BCC'].split(',')
        bcc = [x.strip() for x in bcc]
        if SELF_ADDRESS in bcc:
            bcc.remove(SELF_ADDRESS)
            del msg['BCC']
            msg['BCC'] = ','.join(bcc)

    dest_document_body = msg.as_string()

    # walk through email body
    for part in msg.walk():
        # read message body
        content_type = part.get_content_type()

        if content_type == 'text' or content_type == 'text/plain':
            dest_document_body = part.get_payload(decode=True)

        # message with text will get higher priority than html
        if content_type == 'text/html' and not dest_document_body:
            dest_document_body = part.get_payload(decode=True)

    email_obj = EmailLog.objects.create(
        sender=msg['From'],
        recipient=msg['To'],
        cc=msg['CC'] or '',
        bcc=msg['BCC'] or '',
        subject=msg['Subject'],
        raw_body=dest_document_body,
        message_id=msg['Message-ID'] or "",
        in_reply_to=msg['In-Reply-To'] or ""
    )

    email_obj.raw_email.save(str(uuid.uuid4()), ContentFile(msg.as_string()))

    # saving attachments
    attaches = get_attachments(msg)
    for (file_name, content, mimetype) in attaches:
        file_name = pytils.translit.translify(unicode(file_name), strict=False)
        file_name = ''.join([c for c in file_name if ord(c) <= 128])
        file_name = file_name or 'bad_name'
        email_attach = EmailAttachment(email_log=email_obj)
        email_attach.attachment.save(file_name, ContentFile(content))

    if ',' in email_obj.recipient:
        # TODO few recipients not implemented
        recipients = email_obj.recipient.split(',')
        for recipient in recipients:
            recipient = recipient.strip()
            if not recipient or not is_project_recipient(recipient):
                continue
            email_obj.recipient = recipient
            msg = send_them(
                email_obj, msg, send_email) or msg
    else:
        msg = send_them(email_obj, msg, send_email)
    return msg


def main(raw_email):
    save_email_object(raw_email, send_email=True)


if __name__ == "__main__":
    email_input = sys.stdin.readlines()
    main(email_input)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'EmailRedirect'
        db.create_table('email_filter_emailredirect', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('email_in', self.gf('django.db.models.fields.EmailField')(unique=True, max_length=250)),
            ('email_redirect', self.gf('django.db.models.fields.EmailField')(max_length=250)),
        ))
        db.send_create_signal('email_filter', ['EmailRedirect'])

        # Adding model 'EmailLog'
        db.create_table('email_filter_emaillog', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('sender', self.gf('django.db.models.fields.EmailField')(max_length=250)),
            ('recipient', self.gf('django.db.models.fields.CharField')(max_length=750)),
            ('subject', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('raw_body', self.gf('django.db.models.fields.TextField')()),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('sent', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal('email_filter', ['EmailLog'])


    def backwards(self, orm):
        
        # Deleting model 'EmailRedirect'
        db.delete_table('email_filter_emailredirect')

        # Deleting model 'EmailLog'
        db.delete_table('email_filter_emaillog')


    models = {
        'email_filter.emaillog': {
            'Meta': {'object_name': 'EmailLog'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'raw_body': ('django.db.models.fields.TextField', [], {}),
            'recipient': ('django.db.models.fields.CharField', [], {'max_length': '750'}),
            'sender': ('django.db.models.fields.EmailField', [], {'max_length': '250'}),
            'sent': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'email_filter.emailredirect': {
            'Meta': {'object_name': 'EmailRedirect'},
            'email_in': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '250'}),
            'email_redirect': ('django.db.models.fields.EmailField', [], {'max_length': '250'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['email_filter']



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'EmailLog.cc'
        db.add_column('email_filter_emaillog', 'cc',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=500, blank=''),
                      keep_default=False)

        # Adding field 'EmailLog.bcc'
        db.add_column('email_filter_emaillog', 'bcc',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=500, blank=''),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'EmailLog.cc'
        db.delete_column('email_filter_emaillog', 'cc')

        # Deleting field 'EmailLog.bcc'
        db.delete_column('email_filter_emaillog', 'bcc')


    models = {
        'email_filter.emailattachment': {
            'Meta': {'object_name': 'EmailAttachment'},
            'attachment': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'email_log': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'attachments'", 'to': "orm['email_filter.EmailLog']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'email_filter.emaillog': {
            'Meta': {'object_name': 'EmailLog'},
            'bcc': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '500', 'blank': "''"}),
            'cc': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '500', 'blank': "''"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_reply_to': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'message_id': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'raw_body': ('django.db.models.fields.TextField', [], {}),
            'raw_email': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True'}),
            'recipient': ('django.db.models.fields.CharField', [], {'max_length': '750'}),
            'sender': ('django.db.models.fields.EmailField', [], {'max_length': '250'}),
            'sent': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'email_filter.emailredirect': {
            'Meta': {'unique_together': "(('email_in', 'email_redirect'),)", 'object_name': 'EmailRedirect'},
            'email_in': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '250'}),
            'email_redirect': ('django.db.models.fields.EmailField', [], {'max_length': '250'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['email_filter']


# -*- coding: utf-8 -*-
from django.contrib import admin
from email_filter.models import EmailLog, EmailRedirect, EmailAttachment


class EmailAttachmentInline(admin.TabularInline):
    model = EmailAttachment


class EmailLogAdmin(admin.ModelAdmin):
    list_display = ['sender', 'recipient', 'subject', 'created']
    search_fields = ['sender', 'recipient', 'subject']
    list_filter = ['created', ]
    inlines = [EmailAttachmentInline]


admin.site.register(EmailLog, EmailLogAdmin)
admin.site.register(EmailAttachment)
admin.site.register(EmailRedirect, admin.ModelAdmin)



# -*- coding: utf-8 -*-
import chardet
import datetime
from functools import partial

from django.conf import settings
from django.db import models

from email.utils import parseaddr


def get_cleaned_email(email):
    return parseaddr(email)[1]


def get_name_from_email(email):
    email = get_cleaned_email(email)
    if '@' in email:
        return email.split('@')[0]
    return ''


def is_local_email(email_address):
    email_servers = getattr(settings, 'EMAIL_SERVER_DOMAINS', ['@42cc.co'])
    for server in email_servers:
        if email_address.endswith(server):
            return True
    return False


def is_project_recipient(recipient):
    """
    Returns True is recipient has valid email (ends with @company.mail.domain.mine)
    """
    email = get_cleaned_email(recipient)
    return is_local_email(email)


class ERManager(models.Manager):
    def get_object_or_None(self, *args, **kwargs):
        try:
            return self.get(*args, **kwargs)
        except self.model.DoesNotExist:
            return None

    def get_or_create_back_email(self, email):
        email_server = getattr(settings, 'EMAIL_SERVER_DOMAINS', ['@42cc.co'])[0]
        return_email = self.get_object_or_None(email_in=email)
        if return_email is None:
            er = self.create(
                email_in=email,
                email_redirect='username@mail.com',
            )
            return_email = '%s__%s%s' % (get_name_from_email(email), er.id, email_server)
            er.email_redirect = return_email
            er.save()
        else:
            return_email = return_email.email_redirect
        return return_email

    def get_original_from_back_email(self, email):
        # be on the safe side, use filter instead of get
        ers = self.filter(email_redirect=email)
        if ers.exists():
            return ers[0].email_in
        return None


class EmailRedirect(models.Model):

    email_in = models.EmailField(max_length=250, unique=True)
    email_redirect = models.EmailField(max_length=250)
    objects = ERManager()

    class Meta:
        unique_together = ('email_in', 'email_redirect')

    def __unicode__(self):
        return '[%s] %s -> %s' % (self.id, self.email_in, self.email_redirect)

    @classmethod
    def get_server_email_domain(cls):
        return getattr(settings, 'EMAIL_SERVER_DOMAINS', ['@42cc.co'])[0]

    def save(self, *args, **kwargs):
        self.email_in = get_cleaned_email(self.email_in)
        super(EmailRedirect, self).save(*args, **kwargs)


def upload_to_date(instance, filename, prefix):
    month_number = int(datetime.date.today().toordinal()/30)
    return '{prefix}/{month_number}/{filename}'.format(
        prefix=prefix, filename=filename, month_number=month_number,
        instance=instance
    )


class EmailLog(models.Model):
    sender = models.EmailField(max_length=250)
    recipient = models.CharField(max_length=750)
    cc = models.CharField(max_length=500, default='', blank='')
    bcc = models.CharField(max_length=500, default='', blank='')
    subject = models.CharField(max_length=255)
    raw_body = models.TextField()
    raw_email = models.FileField(upload_to=partial(upload_to_date, prefix='files/emails'), null=True)
    message_id = models.CharField(max_length=255, default='', blank=True)
    in_reply_to = models.CharField(max_length=255, default='', blank=True)

    created = models.DateTimeField(auto_now_add=True)
    sent = models.DateTimeField(blank=True, null=True)

    def __unicode__(self):
        return '[%s] %s' % (self.id, self.subject)

    def save(self, *args, **kwargs):
        decode = kwargs.get('decode', True)
        if decode and self.raw_body and not isinstance(self.raw_body, unicode):
            for body_charset in 'koi8-r', 'latin-1', 'utf-8', 'ascii':
                try:
                    self.raw_body = self.raw_body.decode(body_charset)
                except Exception:
                    pass
                else:
                    break
            # self.raw_body = self.raw_body.decode(
            #     chardet.detect(self.raw_body)['encoding'])
        super(EmailLog, self).save(*args, **kwargs)


class EmailAttachment(models.Model):
    attachment = models.FileField(upload_to=partial(upload_to_date, prefix='files/email_attaches'), null=False)
    email_log = models.ForeignKey(EmailLog, related_name='attachments')



# -*- coding: utf-8 -*-
from setuptools import setup


def get_version(fname='email_filter/__init__.py'):
    with open(fname) as f:
        for line in f:
            if line.startswith('__version__'):
                return eval(line.split('=')[-1])

setup(
    name='email-filter',
    version=get_version(),
    packages=['email_filter', 'email_filter.migrations'],
    author='42 Coffee Cups',
    author_email='email_filter@42cc.co',
    url='https://kavahq.com/42-coffee-cups/project/42-emailfilter/',
    license='BSD(?) licence, see LICENCE.txt',
    description=(
        'Adds the script, which process given emails to proxy them, '
        'replacing real email addresses by company\'s local ones taken from '
        'the contrib.auth module.'),
    long_description=open('README.rst').read(),
    scripts=['email_filter/mail_route.py'],
    zip_safe=False,
    install_requires=[
        "Django >= 1.2.5",
        "django-annoying==0.7.6",
        "chardet==2.2.1",
        "south>= 0.7.3",
        "pytils==0.3",
    ],

)



# -*- coding: utf-8 -*-
from django.dispatch import Signal

mail_for_unknown_user = Signal(
    providing_args=[
        "instance", "original_msg", "mail_sender_email", "send_email", "smtp_sendmail"
    ]
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'EmailLog.raw_email'
        db.add_column(u'email_filter_emaillog', 'raw_email', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True), keep_default=False)

        # Adding field 'EmailLog.message_id'
        db.add_column(u'email_filter_emaillog', 'message_id', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True), keep_default=False)

        # Adding field 'EmailLog.in_reply_to'
        db.add_column(u'email_filter_emaillog', 'in_reply_to', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True), keep_default=False)

        # Adding unique constraint on 'EmailRedirect', fields ['email_in', 'email_redirect']
        db.create_unique(u'email_filter_emailredirect', ['email_in', 'email_redirect'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'EmailRedirect', fields ['email_in', 'email_redirect']
        db.delete_unique(u'email_filter_emailredirect', ['email_in', 'email_redirect'])

        # Deleting field 'EmailLog.raw_email'
        db.delete_column(u'email_filter_emaillog', 'raw_email')

        # Deleting field 'EmailLog.message_id'
        db.delete_column(u'email_filter_emaillog', 'message_id')

        # Deleting field 'EmailLog.in_reply_to'
        db.delete_column(u'email_filter_emaillog', 'in_reply_to')


    models = {
        u'email_filter.emaillog': {
            'Meta': {'object_name': 'EmailLog'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_reply_to': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'message_id': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'raw_body': ('django.db.models.fields.TextField', [], {}),
            'raw_email': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True'}),
            'recipient': ('django.db.models.fields.CharField', [], {'max_length': '750'}),
            'sender': ('django.db.models.fields.EmailField', [], {'max_length': '250'}),
            'sent': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'email_filter.emailredirect': {
            'Meta': {'unique_together': "(('email_in', 'email_redirect'),)", 'object_name': 'EmailRedirect'},
            'email_in': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '250'}),
            'email_redirect': ('django.db.models.fields.EmailField', [], {'max_length': '250'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['email_filter']
